@@ -1,6 +1,6 @@
 /*
     scardcontrol.c: sample code to use/test SCardControl() API
-    Copyright (C) 2004-2008   Ludovic Rousseau
+    Copyright (C) 2004-2009   Ludovic Rousseau
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: scardcontrol.c 2974 2008-05-28 18:32:52Z rousseau $
+ * $Id: scardcontrol.c 4341 2009-07-21 17:54:47Z rousseau $
  */
 
 #include <stdio.h>
@@ -35,8 +35,9 @@
 #endif
 #include <reader.h>
 
-#undef VERIFY_PIN
+#define VERIFY_PIN
 #define MODIFY_PIN
+#undef GET_GEMPC_FIRMWARE
 
 #ifndef TRUE
 #define TRUE 1
@@ -80,6 +81,8 @@ int main(int argc, char *argv[])
 	DWORD send_length, length;
 	DWORD verify_ioctl = 0;
 	DWORD modify_ioctl = 0;
+	DWORD pin_properties_ioctl = 0;
+	DWORD mct_readerdirect_ioctl = 0;
 	SCARD_IO_REQUEST pioRecvPci;
  	SCARD_IO_REQUEST pioSendPci;
 	PCSC_TLV_STRUCTURE *pcsc_tlv;
@@ -92,9 +95,10 @@ int main(int argc, char *argv[])
 #ifdef MODIFY_PIN
 	PIN_MODIFY_STRUCTURE *pin_modify;
 #endif
+	char secoder_info[] = { 0x20, 0x70, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00 };
 
 	printf("SCardControl sample code\n");
-	printf("V 1.3 2004-2008, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
+	printf("V 1.3 © 2004-2009, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
 
 	printf("\nTHIS PROGRAM IS NOT DESIGNED AS A TESTING TOOL!\n");
 	printf("Do NOT use it unless you really know what you do.\n\n");
@@ -148,6 +152,7 @@ int main(int argc, char *argv[])
 	/* fill the readers table */
 	nbReaders = 0;
 	ptr = mszReaders;
+	printf("Available readers (use command line argument to select)\n");
 	while (*ptr != '\0')
 	{
 		printf("%d: %s\n", nbReaders, ptr);
@@ -155,6 +160,7 @@ int main(int argc, char *argv[])
 		ptr += strlen(ptr)+1;
 		nbReaders++;
 	}
+	printf("\n");
 
 	if (argc > 1)
 	{
@@ -170,11 +176,13 @@ int main(int argc, char *argv[])
 
 	/* connect to a reader (even without a card) */
 	dwActiveProtocol = -1;
+	printf("Using reader: %s\n", readers[reader_nb]);
 	rv = SCardConnect(hContext, readers[reader_nb], SCARD_SHARE_DIRECT,
 		SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1, &hCard, &dwActiveProtocol);
 	printf(" Protocol: %ld\n", dwActiveProtocol);
 	PCSC_ERROR_EXIT(rv, "SCardConnect")
 
+#ifdef GET_GEMPC_FIRMWARE
 	/* get GemPC firmware */
 	printf(" Get GemPC Firmware\n");
 
@@ -192,30 +200,12 @@ int main(int argc, char *argv[])
 	printf(" Firmware: %s (length %ld bytes)\n", bRecvBuffer, length);
 
 	PCSC_ERROR_CONT(rv, "SCardControl")
-
-	/* get card status */
-	dwAtrLen = sizeof(pbAtr);
-	dwReaderLen = sizeof(pbReader);
-	rv = SCardStatus(hCard, pbReader, &dwReaderLen, &dwState, &dwProt,
-		pbAtr, &dwAtrLen);
-	printf(" Reader: %s (length %ld bytes)\n", pbReader, dwReaderLen);
-	printf(" State: 0x%04lX\n", dwState);
-	printf(" Prot: %ld\n", dwProt);
-	printf(" ATR (length %ld bytes):", dwAtrLen);
-	for (i=0; i<dwAtrLen; i++)
-		printf(" %02X", pbAtr[i]);
-	printf("\n");
-	PCSC_ERROR_CONT(rv, "SCardStatus")
-
-	if (dwState & SCARD_ABSENT)
-	{
-		printf("No card inserted\n");
-		goto end;
-	}
+#endif
 
 	/* does the reader support PIN verification? */
 	rv = SCardControl(hCard, CM_IOCTL_GET_FEATURE_REQUEST, NULL, 0,
 		bRecvBuffer, sizeof(bRecvBuffer), &length);
+	PCSC_ERROR_EXIT(rv, "SCardControl")
 
 	printf(" TLV (%ld): ", length);
 	for (i=0; i<length; i++)
@@ -236,10 +226,41 @@ int main(int argc, char *argv[])
 	pcsc_tlv = (PCSC_TLV_STRUCTURE *)bRecvBuffer;
 	for (i = 0; i < length; i++)
 	{
-		if (pcsc_tlv[i].tag == FEATURE_VERIFY_PIN_DIRECT)
-			verify_ioctl = ntohl(pcsc_tlv[i].value);
-		if (pcsc_tlv[i].tag == FEATURE_MODIFY_PIN_DIRECT)
-			modify_ioctl = ntohl(pcsc_tlv[i].value);
+		switch (pcsc_tlv[i].tag)
+		{
+			case FEATURE_VERIFY_PIN_DIRECT:
+				printf("Reader supports FEATURE_VERIFY_PIN_DIRECT\n");
+				verify_ioctl = ntohl(pcsc_tlv[i].value);
+				break;
+			case FEATURE_MODIFY_PIN_DIRECT:
+				printf("Reader supports FEATURE_MODIFY_PIN_DIRECT\n");
+				modify_ioctl = ntohl(pcsc_tlv[i].value);
+				break;
+			case FEATURE_IFD_PIN_PROPERTIES:
+				printf("Reader supports FEATURE_IFD_PIN_PROPERTIES\n");
+				pin_properties_ioctl = ntohl(pcsc_tlv[i].value);
+				break;
+			case FEATURE_MCT_READERDIRECT:
+				printf("Reader supports FEATURE_MCT_READERDIRECT\n");
+				mct_readerdirect_ioctl = ntohl(pcsc_tlv[i].value);
+				break;
+			default:
+				printf("Can't parse tag: 0x%02X\n", pcsc_tlv[i].tag);
+		}
+	}
+	printf("\n");
+
+	if (mct_readerdirect_ioctl)
+	{
+		rv = SCardControl(hCard, mct_readerdirect_ioctl, secoder_info,
+			sizeof(secoder_info), bRecvBuffer, sizeof(bRecvBuffer), &length);
+		PCSC_ERROR_CONT(rv, "SCardControl(MCT_READERDIRECT)")
+
+		printf("MCT_READERDIRECT (%ld): ", length);
+		for (i=0; i<length; i++)
+			printf("%02X ", bRecvBuffer[i]);
+		printf("\n");
+
 	}
 
 	if (0 == verify_ioctl)
@@ -249,6 +270,26 @@ int main(int argc, char *argv[])
 		goto end;
 	}
 
+	/* get card status */
+	dwAtrLen = sizeof(pbAtr);
+	dwReaderLen = sizeof(pbReader);
+	rv = SCardStatus(hCard, pbReader, &dwReaderLen, &dwState, &dwProt,
+		pbAtr, &dwAtrLen);
+	printf(" Reader: %s (length %ld bytes)\n", pbReader, dwReaderLen);
+	printf(" State: 0x%04lX\n", dwState);
+	printf(" Prot: %ld\n", dwProt);
+	printf(" ATR (length %ld bytes):", dwAtrLen);
+	for (i=0; i<dwAtrLen; i++)
+		printf(" %02X", pbAtr[i]);
+	printf("\n");
+	PCSC_ERROR_CONT(rv, "SCardStatus")
+
+	if (dwState & SCARD_ABSENT)
+	{
+		printf("No card inserted\n");
+		goto end;
+	}
+
 	/* connect to a reader (even without a card) */
 	dwActiveProtocol = -1;
 	rv = SCardReconnect(hCard, SCARD_SHARE_SHARED,
@@ -297,7 +338,7 @@ int main(int argc, char *argv[])
 	printf(" Secure verify PIN\n");
 	pin_verify = (PIN_VERIFY_STRUCTURE *)bSendBuffer;
 
-	/* PC/SC v2.0.2 Part 10 PIN verification data structure */
+	/* PC/SC v2.02.05 Part 10 PIN verification data structure */
 	pin_verify -> bTimerOut = 0x00;
 	pin_verify -> bTimerOut2 = 0x00;
 	pin_verify -> bmFormatString = 0x82;
@@ -430,7 +471,7 @@ int main(int argc, char *argv[])
 	 * bConfirmPIN = 1, bNumberMessage = 2: "New Pin" "Confirm Pin"
 	 * bConfirmPIN = 0, bNumberMessage = 1: "New Pin"
 	 */
-	/* PC/SC v2.0.2 Part 10 PIN verification data structure */
+	/* PC/SC v2.02.05 Part 10 PIN modification data structure */
 	pin_modify -> bTimerOut = 0x00;
 	pin_modify -> bTimerOut2 = 0x00;
 	pin_modify -> bmFormatString = 0x82;
