@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.c 3280 2009-01-13 19:30:21Z rousseau $
+ * $Id: ccid.c 4346 2009-07-28 13:39:37Z rousseau $
  */
 
 #include <stdio.h>
@@ -37,12 +37,13 @@
 
 /*****************************************************************************
  *
- *					ccid_open_hack
+ *					ccid_open_hack_pre
  *
  ****************************************************************************/
-int ccid_open_hack(unsigned int reader_index)
+int ccid_open_hack_pre(unsigned int reader_index)
 {
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	int doInterruptRead = 1;
 
 	switch (ccid_descriptor->readerID)
 	{
@@ -52,6 +53,77 @@ int ccid_open_hack(unsigned int reader_index)
 			ccid_descriptor->dwFeatures |= CCID_CLASS_TPDU;
 			break;
 
+		case MYSMARTPAD:
+			ccid_descriptor->dwMaxIFSD = 254;
+			break;
+
+		case CL1356D:
+			/* the firmware needs some time to initialize */
+			(void)sleep(1);
+			ccid_descriptor->readTimeout = 60; /* 60 seconds */
+			break;
+
+		case KOBIL_TRIBANK:
+			/* the InterruptRead does not timeout (on Mac OS X) */
+			doInterruptRead = 0;
+			break;
+	}
+
+	/* CCID */
+	if (doInterruptRead && (0 == ccid_descriptor->bInterfaceProtocol))
+	{
+#ifndef TWIN_SERIAL
+		/* just wait for 10ms in case a notification is in the pipe */
+		(void)InterruptRead(reader_index, 10);
+#endif
+	}
+
+	/* ICCD type A */
+	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	{
+		unsigned char tmp[MAX_ATR_SIZE];
+		unsigned int n = sizeof(tmp);
+
+		DEBUG_COMM("ICCD type A");
+		(void)CmdPowerOff(reader_index);
+		(void)CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
+		(void)CmdPowerOff(reader_index);
+	}
+
+	/* ICCD type B */
+	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	{
+		unsigned char tmp[MAX_ATR_SIZE];
+		unsigned int n = sizeof(tmp);
+
+		DEBUG_COMM("ICCD type B");
+		if (CCID_CLASS_SHORT_APDU ==
+			(ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK))
+		{
+			/* use the extended APDU comm alogorithm */
+			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
+			ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
+		}
+
+		(void)CmdPowerOff(reader_index);
+		(void)CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
+		(void)CmdPowerOff(reader_index);
+	}
+
+	return 0;
+} /* ccid_open_hack_pre */
+
+/*****************************************************************************
+ *
+ *					ccid_open_hack_post
+ *
+ ****************************************************************************/
+int ccid_open_hack_post(unsigned int reader_index)
+{
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+
+	switch (ccid_descriptor->readerID)
+	{
 		case GEMPCKEY:
 		case GEMPCTWIN:
 			/* Reader announces TPDU but can do APDU */
@@ -272,58 +344,10 @@ int ccid_open_hack(unsigned int reader_index)
 			}
 			break;
 #endif
-
-		case MYSMARTPAD:
-			ccid_descriptor->dwMaxIFSD = 254;
-			break;
-
-		case CL1356D:
-			/* the firmware needs some time to initialize */
-			(void)sleep(1);
-			ccid_descriptor->readTimeout = 60; /* 60 seconds */
-			break;
-
-		case SEG:
-#ifndef TWIN_SERIAL
-			(void)InterruptRead(reader_index);
-#endif
-			break;
-	}
-
-	/* ICCD type A */
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
-	{
-		unsigned char tmp[MAX_ATR_SIZE];
-		unsigned int n = sizeof(tmp);
-
-		DEBUG_COMM("ICCD type A");
-		(void)CmdPowerOff(reader_index);
-		(void)CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
-		(void)CmdPowerOff(reader_index);
-	}
-
-	/* ICCD type B */
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
-	{
-		unsigned char tmp[MAX_ATR_SIZE];
-		unsigned int n = sizeof(tmp);
-
-		DEBUG_COMM("ICCD type B");
-		if (CCID_CLASS_SHORT_APDU ==
-			(ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK))
-		{
-			/* use the extended APDU comm alogorithm */
-			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
-			ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
-		}
-
-		(void)CmdPowerOff(reader_index);
-		(void)CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
-		(void)CmdPowerOff(reader_index);
 	}
 
 	return 0;
-} /* ccid_open_hack */
+} /* ccid_open_hack_post */
 
 /*****************************************************************************
  *
