@@ -1,6 +1,6 @@
 /*
     ccid.h: CCID structures
-    Copyright (C) 2003   Ludovic Rousseau
+    Copyright (C) 2003-2009   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.h 3207 2008-11-18 15:02:50Z rousseau $
+ * $Id: ccid.h 4280 2009-06-26 14:58:23Z rousseau $
  */
 
 typedef struct
@@ -53,6 +53,11 @@ typedef struct
 	 * PIN support of the reader (directly from Class Descriptor)
 	 */
 	char bPINSupport;
+	
+	/*
+	 * Display dimensions of the reader (directly from Class Descriptor)
+	 */
+	unsigned int wLcdLayout;
 
 	/*
 	 * Default Clock
@@ -108,6 +113,14 @@ typedef struct
 	 * of IFD_ICC_PRESENT
 	 */
 	int dwSlotStatus;
+
+	/*
+	 * bVoltageSupport (bit field)
+	 * 1 = 5.0V
+	 * 2 = 3.0V
+	 * 4 = 1.8V
+	 */
+	int bVoltageSupport;
 } _ccid_descriptor;
 
 /* Features from dwFeatures */
@@ -147,6 +160,8 @@ typedef struct
 #define GEMPCPINPAD 0x08E63478
 #define GEMCORESIMPRO 0x08E63480
 #define GEMCOREPOSPRO 0x08E63479
+#define GEMALTOPROXDU 0x08E65503
+#define GEMALTOPROXSU 0x08E65504
 #define CARDMAN3121	0x076B3021
 #define LTC31		0x07830003
 #define SCR331DI	0x04E65111
@@ -163,6 +178,8 @@ typedef struct
 #define REINER_SCT	0x0C4B0300 
 #define SEG			0x08E68000
 #define BLUDRIVEII_CCID	0x1B0E1078
+#define DELLSCRK    0x413C2101
+#define KOBIL_TRIBANK	0x0D463010
 
 /*
  * The O2Micro OZ776S reader has a wrong USB descriptor
@@ -192,7 +209,8 @@ typedef struct
 #define VOLTAGE_3V 2
 #define VOLTAGE_1_8V 3
 
-int ccid_open_hack(unsigned int reader_index);
+int ccid_open_hack_pre(unsigned int reader_index);
+int ccid_open_hack_post(unsigned int reader_index);
 void ccid_error(int error, const char *file, int line, const char *function);
 _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index);
 
