@@ -1,6 +1,6 @@
 /*
     ccid_ifdhandler.h: non-generic ifdhandler functions
-    Copyright (C) 2004   Ludovic Rousseau
+    Copyright (C) 2004-2009   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_ifdhandler.h 2974 2008-05-28 18:32:52Z rousseau $
+ * $Id: ccid_ifdhandler.h 4279 2009-06-26 14:58:03Z rousseau $
  */
 
 #ifndef _ccid_ifd_handler_h_
@@ -31,6 +31,13 @@
 	SCARD_CTL_CODE(FEATURE_VERIFY_PIN_DIRECT + CLASS2_IOCTL_MAGIC)
 #define IOCTL_FEATURE_MODIFY_PIN_DIRECT \
 	SCARD_CTL_CODE(FEATURE_MODIFY_PIN_DIRECT + CLASS2_IOCTL_MAGIC)
+#define IOCTL_FEATURE_MCT_READERDIRECT \
+	SCARD_CTL_CODE(FEATURE_MCT_READERDIRECT + CLASS2_IOCTL_MAGIC)
+
+#ifdef FEATURE_IFD_PIN_PROPERTIES
+#define IOCTL_FEATURE_IFD_PIN_PROPERTIES \
+	SCARD_CTL_CODE(FEATURE_IFD_PIN_PROPERTIES + CLASS2_IOCTL_MAGIC)
+#endif
 
 #define DRIVER_OPTION_CCID_EXCHANGE_AUTHORIZED 1
 #define DRIVER_OPTION_GEMPC_TWIN_KEY_APDU 2
