@@ -1,6 +1,6 @@
 /*
     ccid_usb.c: USB access routines using the libusb library
-    Copyright (C) 2003-2008   Ludovic Rousseau
+    Copyright (C) 2003-2009   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c 3027 2008-06-26 13:42:22Z rousseau $
+ * $Id: ccid_usb.c 4346 2009-07-28 13:39:37Z rousseau $
  */
 
 #define __CCID_USB__
@@ -84,10 +84,10 @@ typedef struct
 /* The _usbDevice structure must be defined before including ccid_usb.h */
 #include "ccid_usb.h"
 
-static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice);
+static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice, int num);
 int ccid_check_firmware(struct usb_device *dev);
 static unsigned int *get_data_rates(unsigned int reader_index,
-	struct usb_device *dev);
+	struct usb_device *dev, int num);
 
 /* ne need to initialize to 0 since it is static */
 static _usbDevice usbDevice[CCID_DRIVER_MAX_READERS];
@@ -114,6 +114,7 @@ static struct _bogus_firmware Bogus_firmwares[] = {
 	{ 0x0DC3, 0x1004, 0x0502 },	/* ASE IIIe USBv2 */
 	{ 0x0DC3, 0x1102, 0x0607 },	/* ASE IIIe KB USB */
 	{ 0x058F, 0x9520, 0x0102 },	/* Alcor AU9520-G */
+	{ 0x072F, 0x2200, 0x0206 }, /* ACS ACR122U-WB-R */
 
 	/* the firmware version is not correct since I do not have received a
 	 * working reader yet */
@@ -151,8 +152,11 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 	char keyValue[TOKEN_MAX_VALUE_SIZE];
 	unsigned int vendorID, productID;
 	char infofile[FILENAME_MAX];
+#ifndef __APPLE__
 	unsigned int device_vendor, device_product;
+#endif
 	char *dirname = NULL, *filename = NULL;
+	int interface_number = -1;
 	static int previous_reader_index = -1;
 
 	DEBUG_COMM3("Reader index: %X, Device: %s", reader_index, device);
@@ -201,14 +205,59 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 				DEBUG_CRITICAL2("can't parse using libusb scheme: %s", device);
 			}
 		}
+
+		/* format usb:%04x/%04x:libhal:%s
+		 * with %s set to
+		 * /org/freedesktop/Hal/devices/usb_device_VID_PID_SERIAL_ifX
+		 * VID is VendorID
+		 * PID is ProductID
+		 * SERIAL is device serial number
+		 * X is the interface number
+		 */
+		if ((dirname = strstr(device, "libhal:")) != NULL)
+		{
+			const char *p;
+
+#define HAL_HEADER "usb_device_"
+
+			/* parse the hal string */
+			if (
+				/* search the last '/' char */
+				(p = strrchr(dirname, '/'))
+
+				/* if the string starts with "usb_device_" we continue */
+				&& (0 == strncmp(++p, HAL_HEADER, sizeof(HAL_HEADER)-1))
+				/* skip the HAL header */
+				&& (p += sizeof(HAL_HEADER)-1)
+
+				/* search the '_' before PID */
+				&& (p = strchr(++p, '_'))
+
+				/* search the '_' before SERIAL */
+				&& (p = strchr(++p, '_'))
+
+				/* search the '_' before ifX */
+				&& (p = strchr(++p, '_'))
+				&& (0 == strncmp(++p, "if", 2))
+			   )
+			{
+				/* convert the interface number */
+				interface_number = atoi(p+2);
+			}
+			else
+				DEBUG_CRITICAL2("can't parse using libhal scheme: %s", device);
+
+			/* dirname was just a temporary variable */
+			dirname = NULL;
+		}
 	}
 #endif
 
 	if (busses == NULL)
 		usb_init();
 
-	usb_find_busses();
-	usb_find_devices();
+	(void)usb_find_busses();
+	(void)usb_find_devices();
 
 	busses = usb_get_busses();
 
@@ -227,7 +276,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 	}
 
 	/* Info.plist full patch filename */
-	snprintf(infofile, sizeof(infofile), "%s/%s/Contents/Info.plist",
+	(void)snprintf(infofile, sizeof(infofile), "%s/%s/Contents/Info.plist",
 		PCSCLITE_HP_DROPDIR, BUNDLE);
 
 	/* general driver info */
@@ -303,7 +352,30 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					int r, already_used;
 					struct usb_interface *usb_interface = NULL;
 					int interface;
+					int num = 0;
 
+#ifdef USE_COMPOSITE_AS_MULTISLOT
+					static int static_interface = 1;
+
+					{
+						/* simulate a composite device as when libhal is
+						 * used */
+						int readerID = (vendorID << 16) + productID;
+
+						if ((GEMALTOPROXDU == readerID)
+							|| (GEMALTOPROXSU == readerID))
+						{
+							if(interface_number >= 0)
+							{
+								DEBUG_CRITICAL("USE_COMPOSITE_AS_MULTISLOT can't be used with libhal");
+								continue;
+							}
+
+							/* the CCID interfaces are 1 and 2 */
+							interface_number = static_interface;
+						}
+					}
+#endif
 					/* is it already opened? */
 					already_used = FALSE;
 
@@ -349,12 +421,16 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 						}
 						else
 						{
-							DEBUG_INFO3("USB device %s/%s already in use."
-								" Checking next one.",
-								bus->dirname, dev->filename);
+							/* if an interface number is given by HAL we
+							 * continue with this device. */
+							if (-1 == interface_number)
+							{
+								DEBUG_INFO3("USB device %s/%s already in use."
+									" Checking next one.",
+									bus->dirname, dev->filename);
+								continue;
+							}
 						}
-
-						continue;
 					}
 
 					DEBUG_COMM3("Trying to open USB bus/device: %s/%s",
@@ -372,35 +448,52 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					/* now we found a free reader and we try to use it */
 					if (dev->config == NULL)
 					{
-						usb_close(dev_handle);
+						(void)usb_close(dev_handle);
 						DEBUG_CRITICAL3("No dev->config found for %s/%s",
 							 bus->dirname, dev->filename);
 						return STATUS_UNSUCCESSFUL;
 					}
 
-					usb_interface = get_ccid_usb_interface(dev);
+again:
+					usb_interface = get_ccid_usb_interface(dev, &num);
 					if (usb_interface == NULL)
 					{
-						usb_close(dev_handle);
-						DEBUG_CRITICAL3("Can't find a CCID interface on %s/%s",
-							bus->dirname, dev->filename);
-						return STATUS_UNSUCCESSFUL;
+						(void)usb_close(dev_handle);
+						if (0 == num)
+							DEBUG_CRITICAL3("Can't find a CCID interface on %s/%s",
+								bus->dirname, dev->filename);
+						interface_number = -1;
+						continue;
 					}
 
 					if (usb_interface->altsetting->extralen != 54)
 					{
-						usb_close(dev_handle);
+						(void)usb_close(dev_handle);
 						DEBUG_CRITICAL4("Extra field for %s/%s has a wrong length: %d", bus->dirname, dev->filename, usb_interface->altsetting->extralen);
 						return STATUS_UNSUCCESSFUL;
 					}
 
 					interface = usb_interface->altsetting->bInterfaceNumber;
+					if (interface_number >= 0 && interface != interface_number)
+					{
+						/* an interface was specified and it is not the
+						 * current one */
+						DEBUG_INFO3("Wrong interface for USB device %s/%s."
+							" Checking next one.", bus->dirname, dev->filename);
+
+						/* check for another CCID interface on the same device */
+						num++;
+
+						goto again;
+					}
+
 					if (usb_claim_interface(dev_handle, interface) < 0)
 					{
-						usb_close(dev_handle);
+						(void)usb_close(dev_handle);
 						DEBUG_CRITICAL4("Can't claim interface %s/%s: %s",
 							bus->dirname, dev->filename, strerror(errno));
-						return STATUS_UNSUCCESSFUL;
+						interface_number = -1;
+						continue;
 					}
 
 					DEBUG_INFO4("Found Vendor/Product: %04X/%04X (%s)",
@@ -412,12 +505,21 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					/* check for firmware bugs */
 					if (ccid_check_firmware(dev))
 					{
-						usb_close(dev_handle);
+						(void)usb_close(dev_handle);
 						return STATUS_UNSUCCESSFUL;
 					}
 
+#ifdef USE_COMPOSITE_AS_MULTISLOT
+					/* use the next interface for the next "slot" */
+					static_interface++;
+
+					/* reset for a next reader */
+					if (static_interface > 2)
+						static_interface = 1;
+#endif
+
 					/* Get Endpoints values*/
-					get_end_points(dev, &usbDevice[reader_index]);
+					(void)get_end_points(dev, &usbDevice[reader_index], num);
 
 					/* store device information */
 					usbDevice[reader_index].handle = dev_handle;
@@ -434,6 +536,9 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 						(dev->descriptor.idVendor << 16) +
 						dev->descriptor.idProduct;
 					usbDevice[reader_index].ccid.dwFeatures = dw2i(usb_interface->altsetting->extra, 40);
+					usbDevice[reader_index].ccid.wLcdLayout =
+						(usb_interface->altsetting->extra[51] << 8) +
+						usb_interface->altsetting->extra[50];
 					usbDevice[reader_index].ccid.bPINSupport = usb_interface->altsetting->extra[52];
 					usbDevice[reader_index].ccid.dwMaxCCIDMessageLength = dw2i(usb_interface->altsetting->extra, 44);
 					usbDevice[reader_index].ccid.dwMaxIFSD = dw2i(usb_interface->altsetting->extra, 28);
@@ -442,10 +547,11 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					usbDevice[reader_index].ccid.bMaxSlotIndex = usb_interface->altsetting->extra[4];
 					usbDevice[reader_index].ccid.bCurrentSlotIndex = 0;
 					usbDevice[reader_index].ccid.readTimeout = DEFAULT_COM_READ_TIMEOUT;
-					usbDevice[reader_index].ccid.arrayOfSupportedDataRates = get_data_rates(reader_index, dev);
+					usbDevice[reader_index].ccid.arrayOfSupportedDataRates = get_data_rates(reader_index, dev, num);
 					usbDevice[reader_index].ccid.bInterfaceProtocol = usb_interface->altsetting->bInterfaceProtocol;
 					usbDevice[reader_index].ccid.bNumEndpoints = usb_interface->altsetting->bNumEndpoints;
 					usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
+					usbDevice[reader_index].ccid.bVoltageSupport = usb_interface->altsetting->extra[5];
 					goto end;
 				}
 			}
@@ -453,7 +559,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 	}
 end:
 	if (usbDevice[reader_index].handle == NULL)
-		return STATUS_UNSUCCESSFUL;
+		return STATUS_NO_SUCH_DEVICE;
 
 	/* memorise the current reader_index so we can detect
 	 * a new OpenUSBByName on a multi slot reader */
@@ -474,7 +580,8 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 	int rv;
 	char debug_header[] = "-> 121234 ";
 
-	sprintf(debug_header, "-> %06X ", (int)reader_index);
+	(void)snprintf(debug_header, sizeof(debug_header), "-> %06X ",
+		(int)reader_index);
 
 	DEBUG_XXD(debug_header, buffer, length);
 
@@ -512,7 +619,8 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 	int duplicate_frame = 0;
 
 read_again:
-	sprintf(debug_header, "<- %06X ", (int)reader_index);
+	(void)snprintf(debug_header, sizeof(debug_header), "<- %06X ",
+		(int)reader_index);
 
 	rv = usb_bulk_read(usbDevice[reader_index].handle,
 		usbDevice[reader_index].bulk_in, (char *)buffer, *length,
@@ -585,11 +693,11 @@ status_t CloseUSB(unsigned int reader_index)
 
 		/* reset so that bSeq starts at 0 again */
 		if (DriverOptions & DRIVER_OPTION_RESET_ON_CLOSE)
-			usb_reset(usbDevice[reader_index].handle);
+			(void)usb_reset(usbDevice[reader_index].handle);
 
-		usb_release_interface(usbDevice[reader_index].handle,
+		(void)usb_release_interface(usbDevice[reader_index].handle,
 			usbDevice[reader_index].interface);
-		usb_close(usbDevice[reader_index].handle);
+		(void)usb_close(usbDevice[reader_index].handle);
 
 		free(usbDevice[reader_index].dirname);
 		free(usbDevice[reader_index].filename);
@@ -621,11 +729,12 @@ _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index)
  *					get_end_points
  *
  ****************************************************************************/
-static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
+static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice,
+	int num)
 {
 	int i;
 	int bEndpointAddress;
-	struct usb_interface *usb_interface = get_ccid_usb_interface(dev);
+	struct usb_interface *usb_interface = get_ccid_usb_interface(dev, &num);
 
 	/*
 	 * 3 Endpoints maximum: Interrupt In, Bulk In, Bulk Out
@@ -660,7 +769,8 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
  *					get_ccid_usb_interface
  *
  ****************************************************************************/
-/*@null@*/ EXTERNAL struct usb_interface * get_ccid_usb_interface(struct usb_device *dev)
+/*@null@*/ EXTERNAL struct usb_interface * get_ccid_usb_interface(
+	struct usb_device *dev, int *num)
 {
 	struct usb_interface *usb_interface = NULL;
 	int i;
@@ -669,7 +779,7 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
 #endif
 
 	/* if multiple interfaces use the first one with CCID class type */
-	for (i=0; dev->config && i<dev->config->bNumInterfaces; i++)
+	for (i = *num; dev->config && i<dev->config->bNumInterfaces; i++)
 	{
 		/* CCID Class? */
 		if (dev->config->interface[i].altsetting->bInterfaceClass == 0xb
@@ -679,6 +789,8 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
 			)
 		{
 			usb_interface = &dev->config->interface[i];
+			/* store the interface number for further reference */
+			*num = i;
 			break;
 		}
 	}
@@ -690,18 +802,19 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
 		|| (REINER_SCT == readerID) || (BLUDRIVEII_CCID == readerID))
 		&& (0 == usb_interface->altsetting->extralen)) /* this is the bug */
 	{
-		for (i=0; i<usb_interface->altsetting->bNumEndpoints; i++)
+		int j;
+		for (j=0; j<usb_interface->altsetting->bNumEndpoints; j++)
 		{
 			/* find the extra[] array */
-			if (54 == usb_interface->altsetting->endpoint[i].extralen)
+			if (54 == usb_interface->altsetting->endpoint[j].extralen)
 			{
 				/* get the extra[] from the endpoint */
 				usb_interface->altsetting->extralen = 54;
 				usb_interface->altsetting->extra =
-					usb_interface->altsetting->endpoint[i].extra;
+					usb_interface->altsetting->endpoint[j].extra;
 				/* avoid double free on close */
-				usb_interface->altsetting->endpoint[i].extra = NULL;
-				usb_interface->altsetting->endpoint[i].extralen = 0;
+				usb_interface->altsetting->endpoint[j].extra = NULL;
+				usb_interface->altsetting->endpoint[j].extralen = 0;
 				break;
 			}
 		}
@@ -724,7 +837,7 @@ int ccid_check_firmware(struct usb_device *dev)
 	for (i=0; i<sizeof(Bogus_firmwares)/sizeof(Bogus_firmwares[0]); i++)
 	{
 		if (dev->descriptor.idVendor != Bogus_firmwares[i].vendor)
-				continue;
+			continue;
 
 		if (dev->descriptor.idProduct != Bogus_firmwares[i].product)
 			continue;
@@ -760,7 +873,7 @@ int ccid_check_firmware(struct usb_device *dev)
  *
  ****************************************************************************/
 static unsigned int *get_data_rates(unsigned int reader_index,
-	struct usb_device *dev)
+	struct usb_device *dev, int num)
 {
 	int n, i, len;
 	unsigned char buffer[256*sizeof(int)];	/* maximum is 256 records */
@@ -792,7 +905,7 @@ static unsigned int *get_data_rates(unsigned int reader_index,
 	n /= sizeof(int);
 
 	/* we do not get the expected number of data rates */
-	len = get_ccid_usb_interface(dev)->altsetting->extra[27]; /* bNumDataRatesSupported */
+	len = get_ccid_usb_interface(dev, &num)->altsetting->extra[27]; /* bNumDataRatesSupported */
 	if ((n != len) && len)
 	{
 		DEBUG_INFO3("Got %d data rates but was expecting %d", n, len);
@@ -852,33 +965,21 @@ int ControlUSB(int reader_index, int requesttype, int request, int value,
  *
  *					InterruptRead
  *
- *
  ****************************************************************************/
-int InterruptRead(int reader_index)
+int InterruptRead(int reader_index, int timeout /* in ms */)
 {
 	int ret;
 	char buffer[8];
-	int timeout = 2*1000; /* 2 seconds */
-	static int hasfailed = FALSE;
 
-	if (hasfailed)
-	{
-		DEBUG_COMM("driver has failed");
-		return 0;
-	}
-
-	DEBUG_COMM("before");
+	DEBUG_PERIODIC2("before (%d)", reader_index);
 	ret = usb_interrupt_read(usbDevice[reader_index].handle,
 		usbDevice[reader_index].interrupt, buffer, sizeof(buffer), timeout);
-	DEBUG_COMM2("after %d", ret);
-
-	if (0 == ret)
-		hasfailed = TRUE;
+	DEBUG_PERIODIC3("after (%d) (%s)", reader_index, usb_strerror());
 
 	if (ret < 0)
 	{
 		/* if usb_interrupt_read() times out we get EILSEQ or EAGAIN */
-		if ((errno != EILSEQ) && (errno != EAGAIN) && (errno != 0))
+		if ((errno != EILSEQ) && (errno != EAGAIN) && (errno != ENODEV) && (errno != 0))
 			DEBUG_COMM4("usb_interrupt_read(%s/%s): %s",
 					usbDevice[reader_index].dirname,
 					usbDevice[reader_index].filename, strerror(errno));
