@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: commands.c 3004 2008-06-16 08:35:10Z rousseau $
+ * $Id: commands.c 4235 2009-05-29 11:42:46Z rousseau $
  */
 
 #include <string.h>
@@ -171,6 +171,28 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 
 	if (ccid_descriptor->dwFeatures & CCID_CLASS_AUTO_VOLTAGE)
 		voltage = 0;	/* automatic voltage selection */
+	else
+	{
+		int bVoltageSupport = ccid_descriptor->bVoltageSupport;
+
+		if ((1 == voltage) && !(bVoltageSupport & 1))
+		{
+			DEBUG_INFO("5V requested but not support by reader");
+			voltage = 2;	/* 3V */
+		}
+
+		if ((2 == voltage) && !(bVoltageSupport & 2))
+		{
+			DEBUG_INFO("3V requested but not support by reader");
+			voltage = 3;	/* 1.8V */
+		}
+
+		if ((3 == voltage) && !(bVoltageSupport & 4))
+		{
+			DEBUG_INFO("1.8V requested but not support by reader");
+			voltage = 0;	/* auto */
+		}
+	}
 
 again:
 	cmd[0] = 0x62; /* IccPowerOn */
@@ -323,6 +345,18 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 		}
 
 	}
+
+	if (DELLSCRK == ccid_descriptor->readerID)
+	{
+		/* the firmware rejects the cases: 01h-FEh and FFh default
+		 * CCID message. The only value supported is 00h (no message) */
+		if (0x00 != TxBuffer[8])
+		{
+			DEBUG_INFO2("Correct bNumberMessage for Dell keyboard (was %d)",
+				TxBuffer[8]);
+			TxBuffer[8] = 0x00;
+		}
+	}
 #endif
 
 	/* T=1 Protocol Management for a TPDU reader */
@@ -351,7 +385,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 			sdata, 3);
 	}
 
-	/* Build a CCID block from a PC/SC V2.1.2 Part 10 block */
+	/* Build a CCID block from a PC/SC V2.02.05 Part 10 block */
 	for (a = 11, b = 0; b < TxLength; b++)
 	{
 		if (1 == b) /* bTimeOut2 field */
@@ -370,7 +404,11 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	}
 
 	/* SPR532 and Case 1 APDU */
-	if ((SPR532 == ccid_descriptor->readerID) && (TxBuffer[15] == 4))
+	if ((SPR532 == ccid_descriptor->readerID)
+		/* bmPINBlockString = 0 => PIN length not inserted in APDU */
+		&& (0 == TxBuffer[3])
+		/* case 1 APDU */
+		&& (4 == TxBuffer[15]))
 	{
 		RESPONSECODE return_value;
 		unsigned char cmd_tmp[] = { 0x80, 0x02, 0x00 };
@@ -382,6 +420,11 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 			res_tmp, &res_length);
 		if (return_value != IFD_SUCCESS)
 			return return_value;
+
+		/* we need to set bSeq again to avoid a "Duplicate frame detected"
+		 * error since the bSeq of CmdEscape is now greater than bSeq set at
+		 * the beginning of this function */
+		cmd[6] = (*ccid_descriptor->pbSeq)++;
 	}
 
 	i2dw(a - 10, cmd + 1);  /* CCID message length */
@@ -390,7 +433,10 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	ccid_descriptor -> readTimeout = max(30, TxBuffer[0]+10);	/* at least 30 seconds */
 
 	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	{
+		ret = IFD_COMMUNICATION_ERROR;
+		goto end;
+	}
 
 	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
 
@@ -415,6 +461,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 		}
 	}
 
+end:
 	ccid_descriptor -> readTimeout = old_read_timeout;
 	return ret;
 } /* SecurePINVerify */
@@ -466,8 +513,9 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 		return IFD_NOT_SUPPORTED;
 	}
 
-	/* Make sure in the beginning if bNumberMessage is valid or not */
-	if (TxBuffer[11] > 3)
+	/* Make sure in the beginning if bNumberMessage is valid or not.
+	 * 0xFF is the default value. */
+	if ((TxBuffer[11] > 3) && (TxBuffer[11] != 0xFF))
 	{
 		DEBUG_INFO2("Wrong bNumberMessage: %d", TxBuffer[11]);
 		return IFD_NOT_SUPPORTED;
@@ -555,7 +603,7 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 			sdata, 3);
 	}
 
-	/* Build a CCID block from a PC/SC V2.1.2 Part 10 block */
+	/* Build a CCID block from a PC/SC V2.02.05 Part 10 block */
 
 	/* Do adjustments as needed - CCID spec is not exact with some
 	 * details in the format of the structure, per-reader adaptions
@@ -610,7 +658,10 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	ccid_descriptor -> readTimeout = max(30, TxBuffer[0]+10);	/* at least 30 seconds */
 
 	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
- 		return IFD_COMMUNICATION_ERROR;
+	{
+ 		ret = IFD_COMMUNICATION_ERROR;
+		goto end;
+	}
 
  	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
 
@@ -635,6 +686,7 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 		}
 	}
 
+end:
 	ccid_descriptor -> readTimeout = old_read_timeout;
 	return ret;
 } /* SecurePINModify */
@@ -653,19 +705,27 @@ RESPONSECODE CmdEscape(unsigned int reader_index,
 	status_t res;
 	unsigned int length_in, length_out;
 	RESPONSECODE return_value = IFD_SUCCESS;
+	int old_read_timeout;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
+	old_read_timeout = ccid_descriptor -> readTimeout;
+	ccid_descriptor -> readTimeout = 30;	/* 30 seconds */
+
 again:
 	/* allocate buffers */
 	length_in = 10 + TxLength;
 	if (NULL == (cmd_in = malloc(length_in)))
-		return IFD_COMMUNICATION_ERROR;
+	{
+		return_value = IFD_COMMUNICATION_ERROR;
+		goto end;
+	}
 
 	length_out = 10 + *RxLength;
 	if (NULL == (cmd_out = malloc(length_out)))
 	{
 		free(cmd_in);
-		return IFD_COMMUNICATION_ERROR;
+		return_value = IFD_COMMUNICATION_ERROR;
+		goto end;
 	}
 
 	cmd_in[0] = 0x6B; /* PC_to_RDR_Escape */
@@ -682,7 +742,8 @@ again:
 	if (res != STATUS_SUCCESS)
 	{
 		free(cmd_out);
-		return IFD_COMMUNICATION_ERROR;
+		return_value = IFD_COMMUNICATION_ERROR;
+		goto end;
 	}
 
 	res = ReadPort(reader_index, &length_out, cmd_out);
@@ -700,13 +761,15 @@ again:
 	if (res != STATUS_SUCCESS)
 	{
 		free(cmd_out);
-		return IFD_COMMUNICATION_ERROR;
+		return_value = IFD_COMMUNICATION_ERROR;
+		goto end;
 	}
 
 	if (length_out < STATUS_OFFSET+1)
 	{
 		DEBUG_CRITICAL2("Not enough data received: %d bytes", length_out);
-		return IFD_COMMUNICATION_ERROR;
+		return_value = IFD_COMMUNICATION_ERROR;
+		goto end;
 	}
 
 	if (cmd_out[STATUS_OFFSET] & CCID_COMMAND_FAILED)
@@ -724,6 +787,8 @@ again:
 
 	free(cmd_out);
 
+end:
+	ccid_descriptor -> readTimeout = old_read_timeout;
 	return return_value;
 } /* Escape */
 
@@ -855,7 +920,7 @@ again_status:
 		if (status[0] & 0x40)
 		{
 			DEBUG_INFO2("Busy: 0x%02X", status[0]);
-			usleep(1000 * 10);
+			(void)usleep(1000 * 10);
 			goto again_status;
 		}
 
@@ -1189,7 +1254,7 @@ time_request_ICCD_B:
 				if (0 == delay)
 					/* host select the delay */
 					delay = 1;
-				usleep(delay * 1000 * 10);
+				(void)usleep(delay * 1000 * 10);
 				goto time_request_ICCD_B;
 			}
 
@@ -1427,7 +1492,10 @@ receive_next_block:
 		 * next PC_to_RDR_XfrBlock command */
 		case 0x10:
 			/* send a nul block */
-			return_value = CCID_Transmit(reader_index, 0, NULL, 0, 0);
+			/* set wLevelParameter to 0010h: empty abData field,
+			 * continuation of response APDU is
+			 * expected in the next RDR_to_PC_DataBlock. */
+			return_value = CCID_Transmit(reader_index, 0, NULL, 0x10, 0);
 			if (return_value != IFD_SUCCESS)
 				return return_value;
 
@@ -1499,7 +1567,7 @@ static RESPONSECODE CmdXfrBlockTPDU_T0(unsigned int reader_index,
  *
  ****************************************************************************/
 static RESPONSECODE T0CmdParsing(unsigned char *cmd, unsigned int cmd_len,
-	unsigned int *exp_len)
+	/*@out@*/ unsigned int *exp_len)
 {
 	*exp_len = 0;
 
