@@ -1,6 +1,6 @@
 /*
     ifdhandler.c: IFDH API
-    Copyright (C) 2003-2008   Ludovic Rousseau
+    Copyright (C) 2003-2009   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/* $Id: ifdhandler.c 3337 2009-03-07 14:30:17Z rousseau $ */
+/* $Id: ifdhandler.c 4346 2009-07-28 13:39:37Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -74,6 +74,7 @@ EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
 {
 	RESPONSECODE return_value = IFD_SUCCESS;
 	int reader_index;
+	status_t ret;
 
 	if (! DebugInitialized)
 		init_driver();
@@ -97,10 +98,14 @@ EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
 	(void)pthread_mutex_lock(&ifdh_context_mutex);
 #endif
 
-	if (OpenPortByName(reader_index, lpcDevice) != STATUS_SUCCESS)
+	ret = OpenPortByName(reader_index, lpcDevice);
+	if (ret != STATUS_SUCCESS)
 	{
 		DEBUG_CRITICAL("failed");
-		return_value = IFD_COMMUNICATION_ERROR;
+		if (STATUS_NO_SUCH_DEVICE == ret)
+			return_value = IFD_NO_SUCH_DEVICE;
+		else
+			return_value = IFD_COMMUNICATION_ERROR;
 
 		/* release the allocated reader_index */
 		ReleaseReaderIndex(reader_index);
@@ -108,7 +113,7 @@ EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
 	else
 	{
 		/* Maybe we have a special treatment for this reader */
-		(void)ccid_open_hack(reader_index);
+		(void)ccid_open_hack_pre(reader_index);
 
 		/* Try to access the reader */
 		/* This "warm up" sequence is sometimes needed when pcscd is
@@ -126,6 +131,9 @@ EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
 			(void)ClosePort(reader_index);
 			ReleaseReaderIndex(reader_index);
 		}
+		else 
+			/* Maybe we have a special treatment for this reader */
+			(void)ccid_open_hack_post(reader_index);
 	}
 
 #ifdef HAVE_PTHREAD
@@ -205,8 +213,30 @@ EXTERNAL RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
 		ReleaseReaderIndex(reader_index);
 	}
 	else
+	{
 		/* Maybe we have a special treatment for this reader */
-		(void)ccid_open_hack(reader_index);
+		(void)ccid_open_hack_pre(reader_index);
+
+		/* Try to access the reader */
+		/* This "warm up" sequence is sometimes needed when pcscd is
+		 * restarted with the reader already connected. We get some
+		 * "usb_bulk_read: Resource temporarily unavailable" on the first
+		 * few tries. It is an empirical hack */
+		if ((IFD_COMMUNICATION_ERROR == IFDHICCPresence(Lun))
+			&& (IFD_COMMUNICATION_ERROR == IFDHICCPresence(Lun))
+			&& (IFD_COMMUNICATION_ERROR == IFDHICCPresence(Lun)))
+		{
+			DEBUG_CRITICAL("failed");
+			return_value = IFD_COMMUNICATION_ERROR;
+
+			/* release the allocated resources */
+			(void)ClosePort(reader_index);
+			ReleaseReaderIndex(reader_index);
+		}
+		else
+			/* Maybe we have a special treatment for this reader */
+			(void)ccid_open_hack_post(reader_index);
+	}
 
 #ifdef HAVE_PTHREAD
 	(void)pthread_mutex_unlock(&ifdh_context_mutex);
@@ -273,7 +303,7 @@ static RESPONSECODE IFDHPolling(DWORD Lun)
 	if (LogLevel & DEBUG_LEVEL_PERIODIC)
 		DEBUG_INFO3("%s (lun: %X)", CcidSlots[reader_index].readerName, Lun);
 
-	ret = InterruptRead(reader_index);
+	ret = InterruptRead(reader_index, 2*1000);	/* 2 seconds */
 	if (ret > 0)
 		return IFD_SUCCESS;
 	if (0 == ret)
@@ -374,6 +404,17 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 			{
 				*Length = 1;
 				*Value = 1 + get_ccid_descriptor(reader_index) -> bMaxSlotIndex;
+#ifdef USE_COMPOSITE_AS_MULTISLOT
+				{
+					/* On MacOS X or Linux+libusb we can simulate a
+					 * composite device with 2 CCID interfaces by a
+					 * multi-slot reader */
+					int readerID =  get_ccid_descriptor(reader_index) -> readerID;
+
+					if ((GEMALTOPROXDU == readerID) || (GEMALTOPROXSU == readerID))
+						*Value = 2;
+				}
+#endif
 				DEBUG_INFO2("Reader supports %d slot(s)", *Value);
 			}
 			break;
@@ -1190,7 +1231,7 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		}
 	}
 
-	/* Implement the PC/SC v2.1.2 Part 10 IOCTL mechanism */
+	/* Implement the PC/SC v2.02.05 Part 10 IOCTL mechanism */
 
 	/* Query for features */
 	if (CM_IOCTL_GET_FEATURE_REQUEST == dwControlCode)
@@ -1198,8 +1239,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		unsigned int iBytesReturned = 0;
 		PCSC_TLV_STRUCTURE *pcsc_tlv = (PCSC_TLV_STRUCTURE *)RxBuffer;
 
-		/* we need room for two records */
-		if (RxLength < 2 * sizeof(PCSC_TLV_STRUCTURE))
+		/* we need room for up to for records */
+		if (RxLength < 4 * sizeof(PCSC_TLV_STRUCTURE))
 			return IFD_COMMUNICATION_ERROR;
 
 		/* We can only support direct verify and/or modify currently */
@@ -1224,10 +1265,52 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 			pcsc_tlv++;
 			iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
 		}
+
+#ifdef FEATURE_IFD_PIN_PROPERTIES
+		/* We can always forward wLcdLayout */
+		pcsc_tlv -> tag = FEATURE_IFD_PIN_PROPERTIES;
+		pcsc_tlv -> length = 0x04; /* always 0x04 */
+		pcsc_tlv -> value = htonl(IOCTL_FEATURE_IFD_PIN_PROPERTIES);
+
+		pcsc_tlv++;
+		iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
+#endif
+
+		if (KOBIL_TRIBANK == get_ccid_descriptor(reader_index) -> readerID)
+		{
+			pcsc_tlv -> tag = FEATURE_MCT_READERDIRECT;
+			pcsc_tlv -> length = 0x04; /* always 0x04 */
+			pcsc_tlv -> value = htonl(IOCTL_FEATURE_MCT_READERDIRECT);
+
+			pcsc_tlv++;
+			iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
+		}
+
 		*pdwBytesReturned = iBytesReturned;
 		return_value = IFD_SUCCESS;
 	}
 
+#ifdef FEATURE_IFD_PIN_PROPERTIES
+	/* Get PIN handling capabilities */
+	if (IOCTL_FEATURE_IFD_PIN_PROPERTIES == dwControlCode)
+	{
+		PIN_PROPERTIES_STRUCTURE *caps = (PIN_PROPERTIES_STRUCTURE *)RxBuffer;
+
+		if (RxLength < sizeof(PIN_PROPERTIES_STRUCTURE))
+			return IFD_COMMUNICATION_ERROR;
+
+		/* Only give the LCD size for now */
+		caps -> wLcdLayout = get_ccid_descriptor(reader_index) -> wLcdLayout;
+		caps -> wLcdMaxCharacters = 0x0000;
+		caps -> wLcdMaxLines = 0x0000; 
+		caps -> bEntryValidationCondition = 0x07; /* Default */
+		caps -> bTimeOut2 = 0x00; /* We do not distinguish bTimeOut from TimeOut2 */
+
+		*pdwBytesReturned = sizeof(*caps);
+		return_value = IFD_SUCCESS;
+	}
+#endif
+
 	/* Verify a PIN, plain CCID */
 	if (IOCTL_FEATURE_VERIFY_PIN_DIRECT == dwControlCode)
 	{
@@ -1250,6 +1333,38 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		*pdwBytesReturned = iBytesReturned;
 	}
 
+	/* MCT: Multifunctional Card Terminal */
+	if (IOCTL_FEATURE_MCT_READERDIRECT == dwControlCode)
+	{
+		if ( (TxBuffer[0] != 0x20)	/* CLA */
+			|| ((TxBuffer[1] & 0xF0) != 0x70)	/* INS */
+			/* valid INS are
+			 * 0x70: SECODER INFO
+			 * 0x71: SECODER SELECT APPLICATION
+			 * 0x72: SECODER APPLICATION ACTIVE
+			 * 0x73: SECODER DATA CONFIRMATION
+			 * 0x74: SECODER PROCESS AUTHENTICATION TOKEN */
+			|| ((TxBuffer[1] & 0x0F) > 4)
+			|| (TxBuffer[2] != 0x00)	/* P1 */
+			|| (TxBuffer[3] != 0x00)	/* P2 */
+			|| (TxBuffer[4] != 0x00)	/* Lind */
+		   )
+		{
+			DEBUG_INFO("MCT Command refused by driver");
+			return_value = IFD_COMMUNICATION_ERROR;
+		}
+		else
+		{
+			unsigned int iBytesReturned;
+
+			/* we just transmit the buffer as a CCID Escape command */
+			iBytesReturned = RxLength;
+			return_value = CmdEscape(reader_index, TxBuffer, TxLength, RxBuffer,
+				&iBytesReturned);
+			*pdwBytesReturned = iBytesReturned;
+		}
+	}
+
 	if (IFD_SUCCESS != return_value)
 		*pdwBytesReturned = 0;
 
